#$Id: schema.tcl,v 1.1.1.1 2005/01/16 00:03:32 pjaol Exp $

proc soap.schema { doc name {results} } {
    set root [$doc documentElement]
    global _schema
    set _schema $doc
    set elementXSD \
	[$root selectNodes \
	     "/xs:schema/xs:element\[@name='$name'\]"]
    set res [soap.schema.parseElements \
		  $elementXSD $name $results]
    return $res
}


proc soap.schema.parseElements { nodes name {results {}} } {

    if {[$nodes hasChildNodes]} {
	foreach node [$nodes childNodes] {
	    
	    set nodeType [$node nodeName]
	    if {$nodeType == "xs:simpleType"} {
		
		return [soap.schema.simpleType $nodes \
			    $name \
			    $results]
	    } else {
		
		return [soap.schema.complexType $nodes\
			    $name\
			    $results]
	    }
	}
    } else {
	
	set type ""
	catch {
	    set type [$node getAttribute type]
	}
	
	if { $type == "xs:simpleType" || [soap.isNaturalType $type]} {
	    return [soap.schema.simpleType $nodes $name $results]
	} else {
	    return [soap.schema.complexType $nodes $name $results $type]
	}
    }
}




proc soap.schema.simpleType { node name {results} } {

    global _soapdoc
    if {[$node hasChildNodes]} {
	set restriction [$node getElementsByTagName xs:restriction]
	set type [$restriction getAttribute base]
    } else {
	set type [$node getAttribute type]
    }
    #could be a simpleType element from
    #a complexType complexContent, where local attribute name
    #overrides parent name, if no name use parent name
    catch {
	set name [$node getAttribute name]
    } err

    set element [$_soapdoc createElement $name]
    $element setAttribute type $type
    $element appendFromList [list "\#text" $results]
    
    return $element
}



proc soap.schema.complexType { node name {results {}} {type {}} } {

    if {[string length $type]} {
	set nodes [$node selectNodes "/xs:complexType\[@name='$type'\]"]
	return [soap.schema.complexTypeContent $nodes $name $results]
    }
    
    $node firstChild child
    return [soap.schema.complexTypeContent $child $name $results]
}



proc soap.schema.complexTypeContent { node name {results {}}} {

    global _soapdoc
    set res [$_soapdoc createElement $name]
    set name [$node nodeName]

    if { $name == "xs:complexType" } {
	
	set content [$node firstChild]
	set body [$content nodeName]
	
	if {$body == "xs:sequence"} {
	    foreach item [$content childNodes] {
		set thisResult [lindex $results 0]
		$res appendChild \
		    [soap.schema.parseElements $item\
			    $name\
			 $thisResult]
		set results [lrange $results 1 end]
	    }
	} elseif { $body == "xs:attribute" } {
	    set atName [$content getAttribute name]
	    $res setAttribute $atName $results
	} elseif { $body == "xs:complexContent" } {
	    soap.schema.complexContent $content $name $results
	}
	
    }

    return $res
}



proc soap.schema.complexContent { node name {results {}}} {
    error "soap.schema.complexContent not implemented yet"
}



proc soap.isNaturalType { type } {

    set valid "xs:string xs:Boolean xs:float xs:double\
               xs:decimal xs:binary xs:integer\
               xs:nonPositiveInteger xs:negativeInteger\
               xs:long xs:short xs:byte xs:nonNegativeInteger\
               xs:unsignedLong xs:unsignedInt xs:unsignedShort \
               xs:unsignedByte xs:positiveInteger xs:date xs:time"

    return [string match "*$type*" $valid]
}
